/*
 * $Header: /home/cvs/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/lib/properties/SPProperty.java,v 1.1 2001/07/17 04:00:53 msmith Exp $
 * $Revision: 1.1 $
 * $Date: 2001/07/17 04:00:53 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui.lib.properties;

import java.lang.reflect.Constructor;
import java.util.StringTokenizer;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.apache.webdav.lib.*;
import org.apache.webdav.ui.util.SPUtils;
import org.apache.util.*;

/**
 * Title:        SPProperty.java
 * Description:  A Property Base Class. It also functions as an adapter to
 *               org.apache.webdav.lib.Property
 * Company:      SpeedLegal Holdings Inc.
 * @author       Jojada J. Tirtowidjojo
 * @version 1.0
 */

public class SPProperty implements Property {

  Property property;

  public static SPProperty factory(Property property) {
    SPProperty theSPProperty=null;

    try {
      //Currently the result of Property.getName() is inconsistent.
      //It may or may not contain namespace. In addition, a property name
      //may have a dash character.
      //converToClassName() removes the namespace and any dash character
      String propName = SPUtils.convertToClassName(property.getName());

      Class propClass = Class.forName(
		  "org.apache.webdav.ui.lib.properties."+propName);
      Constructor constructor = propClass.getConstructor(
                  new Class[] {Property.class});
      theSPProperty = (SPProperty) constructor.newInstance(
                  new Object[] {property});
    } catch (Exception e) {
      theSPProperty = new SPProperty(property);
    }

    return theSPProperty;
  }

  protected SPProperty(Property property)  {
    this.property = property;
  }

  public String getName() {
    return property.getName();
  }

  public String getLocalName()  {
    return property.getLocalName();
  }

  public String getNamespaceURI() {
    return property.getNamespaceURI();
  }

  public Element getElement() {
    return property.getElement();
  }

  public String getPropertyAsString() {
    return property.getPropertyAsString();
  }

  public int getStatusCode()  {
    return property.getStatusCode();
  }

  public String getOwningURL()  {
    return property.getOwningURL();
  }

  public String getFieldValue(String userName, String fieldName) {
    String theFieldValue = null;

    Element thisPropElement = getElement();

    NodeList list = thisPropElement.getElementsByTagName(fieldName);
    if (list.getLength() == 1) { //if it is a field
      Element field = (Element) list.item(0);

      if ( (field.getTagName().equals("locktoken")) ||
           (field.getTagName().equals("owner")) ) {
        NodeList list2 = field.getElementsByTagName("href");

        if (list2.getLength() == 1) { //if exists
          theFieldValue = DOMUtils.getTextValue(list2.item(0));
        }
      } else  {
        theFieldValue = DOMUtils.getTextValue(field);
        if (theFieldValue.equals("")) { //if empty
          theFieldValue = field.getNodeName();
        }
      }
    } //if (list.getLength() == 1);

    return theFieldValue;
  }

  public Property getPropertyHandle()  {
    return property;
  }

}// End of SPProperty class







