/*
 * $Header: /home/cvs/jakarta-slide/src/stores/slidestore/reference/DefaultStore.java,v 1.1 2001/07/25 18:04:09 dirkv Exp $
 * $Revision: 1.1 $
 * $Date: 2001/07/25 18:04:09 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package slidestore.reference;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import java.io.FileWriter;
import java.io.IOException;
import org.apache.slide.common.*;
import org.apache.slide.store.*;
import org.apache.slide.structure.*;
import org.apache.slide.security.*;
import org.apache.slide.lock.*;
import org.apache.slide.content.*;
import org.apache.slide.util.logger.Logger;

/**
 * Default store, extends MemoryDescriptorsStore 
 * and delegates content to FileContentStore.
 *
 * @author Dirk Verbeeck
 * @version $Revision: 1.1 $
 */
public class DefaultStore extends MemoryDescriptorsStore implements LockStore, NodeStore, RevisionDescriptorsStore, RevisionDescriptorStore, SecurityStore, ContentStore {
    
    
    // ----------------------------------------------------- Instance Variables
	
	private FileContentStore contentStore=new FileContentStore();
    
	// ---------------------------------------------------- ServiceImpl Methods
	
	/**
	 * Read parameters.
	 *
	 * @param parameters Hashtable containing the parameters' name
	 * and associated value
	 */
	public synchronized void setParameters(Hashtable parameters)
	    throws ServiceParameterErrorException,
	    ServiceParameterMissingException 
	{
		super.setParameters(parameters);
		contentStore.setParameters(parameters);
	}
	
	
	/**
	 * Connects to store.
	 *
	 * @exception VersionException
	 */
	public synchronized void connect()
	    throws ServiceConnectionFailedException {
		super.connect();
		contentStore.connect();
	}
	
	
	/**
	 * Disconnects from content store.
	 *
	 * @exception ServiceDisconnectionFailedException
	 */
	public synchronized void disconnect()
	    throws ServiceDisconnectionFailedException {
		super.disconnect();
		contentStore.disconnect();
	}
	
	
	/**
	 * Initializes content store.
	 *
	 * @exception ServiceInitializationFailedException Throws an exception
	 * if the store has already been initialized before
	 */
	public synchronized void initialize(NamespaceAccessToken token)
	    throws ServiceInitializationFailedException {
		super.initialize(token);
		contentStore.initialize(token);
	}
	
	
	/**
	 * Deletes content store.
	 *
	 * @exception ServiceResetFailedException
	 */
	public void reset()
	    throws ServiceResetFailedException {
		super.reset();
		contentStore.reset();
	}
	
	
	/**
	 * This function tells whether or not the service is connected.
	 *
	 * @return boolean true if we are connected
	 * @exception ServiceAccessException Service access error
	 */
	public boolean isConnected()
	    throws ServiceAccessException {
		return super.isConnected() && contentStore.isConnected();
	}
	
	
	/**
	 * Commits transaction
	 *
	 * @exception ServiceAccessException Unspecified service access error
	 */
	public synchronized void commit()
	    throws ServiceAccessException {
		super.commit();
		contentStore.commit();
	}
	
	// --------------------------------------------------- ContentStore Methods

	// ContentStore
    public NodeRevisionContent retrieveRevisionContent 
	    (Uri uri, NodeRevisionDescriptor revisionDescriptor) 
		throws ServiceAccessException, RevisionNotFoundException 
	{
		return contentStore.retrieveRevisionContent(uri, revisionDescriptor);
    }
    
    public void createRevisionContent
        (Uri uri, NodeRevisionDescriptor revisionDescriptor,
         NodeRevisionContent revisionContent)
        throws ServiceAccessException, RevisionAlreadyExistException
    {
    	contentStore.createRevisionContent(uri, revisionDescriptor, revisionContent);
    }
    
    public void storeRevisionContent
        (Uri uri, NodeRevisionDescriptor revisionDescriptor,
         NodeRevisionContent revisionContent)
        throws ServiceAccessException, RevisionNotFoundException
    {
		contentStore.storeRevisionContent(uri, revisionDescriptor, revisionContent);
    }
    
    public void removeRevisionContent(Uri uri, NodeRevisionDescriptor revisionDescriptor)
        throws ServiceAccessException
    {
		contentStore.removeRevisionContent(uri, revisionDescriptor);
    }
}
