/*
 * $Header: /home/cvs/jakarta-slide/src/webdav/server/org/apache/slide/webdav/logger/XHttpServletResponseFacade23.java,v 1.2 2001/08/25 13:00:38 dirkv Exp $
 * $Revision: 1.2 $
 * $Date: 2001/08/25 13:00:38 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */


package org.apache.slide.webdav.logger;

import java.io.Writer;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.util.Locale;
import java.util.Enumeration;
import java.util.Vector;
import javax.servlet.http.Cookie;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;
import javax.servlet.http.HttpServletRequest;

import org.apache.slide.common.Domain;

/**
 * This class supports additional set-methods and a re-readable
 * inputstream to the interface javax.servlet.http.HttpServletResponse.
 *
 * @author    Hardy.Kiefer@softwareag.com
 *            Christopher Lenz (cmlenz at apache.org)
 *
 * @version   0.1
 *
 * @invariant (inputStream != null)
 *
 * @see       javax.servlet.http.HttpServletResponse
 *
 */
public class XHttpServletResponseFacade extends HttpServletResponseWrapper
{

    
    public static final String DEFAULT_CHAR_ENCODING = "8859_1";

    
    // default implementation will just append everything here
    StringBuffer body=null;

    protected HttpServletRequest request;
    
    public void setRequest(HttpServletRequest request) {
        this.request = request;
    }

    public HttpServletRequest getRequest() {
        return request;
    }
    
    /**
     * true - if outputstream can write using a stream; false otherwise.
     */
    private boolean usingStream = false;
    
    /**
     * true - if outputstream can write using a writer; false otherwise.
     */
    private boolean usingWriter = false;

    /**
     * output stream used for building the writer.
     *
     * @see javax.servlet.ServletOutputStream
     */
    XServletOutputStreamFacade osFacade = null;
    
    /**
     * writer which uses the output stream.
     *
     * @see java.io.PrintWriter
     */
    PrintWriter writer = null;

    /**
     * response headers
     */
    Vector responseHeaders = new Vector();
    public Enumeration getResponseHeaders() {
        return responseHeaders.elements();
    }
    
    
    /**
     * This constructor creates an re-writable HttpServletRespmse.
     *
     * @pre        (response != null)
     * @post
     *
     * @param      response   HttpServletResponse
     *
     * @time
     * @space
     */
    public XHttpServletResponseFacade(HttpServletResponse response) {
        super(response);
		Domain.debug("Create XHttpServletResponseFacade");
    }


    // -------------------- Public methods --------------------

    /**
     * Adds the specified cookie to the response.
     *
     * @param      cookie - the Cookie to return to the client
     *
     * @return     none
     */
    public void addCookie(javax.servlet.http.Cookie cookie) {
        responseHeaders.add(
            new XResponseHeader(
                "Cookie",
                cookie.getName() + " " +
                cookie.getValue() + " " +
                cookie.getDomain()
                )
            );
        super.addCookie(cookie);
    }
	
    /**
     * Returns a ServletOutputStream suitable for writing binary data in the response.
     *
     * @return     a ServletOutputStream for writing binary data
     */
    public ServletOutputStream getOutputStream()
        throws IOException {
        Domain.debug("ENTER: XHttpServletResponseFacade:getOutputStream()");
        if ( usingWriter ) {
            throw new IllegalStateException("Writer is already being used for this response");
        }
        usingStream = true;

        if( osFacade == null ) {
            osFacade = new XServletOutputStreamFacade(
                super.getOutputStream(),
                super.getCharacterEncoding());
        }
            
        Domain.debug("LEAVE: XHttpServletResponseFacade:getOutputStream()");
        return osFacade;
    }

    /**
     * Returns a PrintWriter object that can send character text to the client.
     *
     * @return     a PrintWriter object that can return character data to the client
     */
    public PrintWriter getWriter() throws IOException {
        Domain.debug("ENTER: XHttpServletResponseFacade:getWriter()");
        if (usingStream) {
            throw new IllegalStateException("OutputStream is already being used for this response");
        }
        usingWriter= true ;

        if( writer == null ) {
            writer = new XServletWriterFacade( super.getWriter() );
        }

        Domain.debug("LEAVE: XHttpServletResponseFacade:getWriter()");
        return writer;
    }

    public PrintWriter getWriter(ServletOutputStream outs) throws IOException {
        Domain.debug("ENTER: XHttpServletResponseFacade:getWriter(ServletOutputStream)");
        if(writer!=null) return writer;
        // it already did all the checkings
    
        //hak started = true;
        usingWriter = true;
    
        //  writer = new XServletWriterFacade( getConverter(outs), this);
        Domain.debug("LEAVE: XHttpServletResponseFacade:getWriter(ServletOutputStream)");
        return writer;
    }

    public Writer getConverter( ServletOutputStream outs ) throws IOException {
        String encoding = getCharacterEncoding();
        OutputStreamWriter osWriter;
        if (encoding == null) {
            // use default platform encoding - is this correct ?
            osWriter = new OutputStreamWriter(outs);
        } else {
            try {
                osWriter = new OutputStreamWriter(outs, encoding);
            } catch (java.io.UnsupportedEncodingException ex) {
                // XXX log it
                System.out.println("Unsuported encoding: " + encoding );
                return new OutputStreamWriter(outs);
            }
        }
        return osWriter;
    }

    /**
     * Sets a response header with the given name and date-value.
     *
     * @param      name - the name of the header to set
     *            value - the assigned date value
     */
    public void setDateHeader(String name, long date) {
        responseHeaders.add(new XResponseHeader(name, String.valueOf(date)));
        super.setDateHeader(name, date);
    }

    /**
     * Adds a response header with the given name and date-value.
     *
     * @param      name - the name of the header to set
     *            value - the additional date value
     */
    public void addDateHeader(String name, long date) {
        responseHeaders.add(new XResponseHeader(name, String.valueOf(date)));
        super.addDateHeader(name, date);
    }

    /**
     * Sets a response header with the given name and value.
     *
     * @param      name - the name of the header
     *            value - the header value
     */
    public void setHeader(String name, String value) {
        responseHeaders.add(new XResponseHeader(name, value));
        super.setHeader(name, value);
    }

    /**
     * Adds a response header with the given name and value.
     *
     * @param      name - the name of the header
     *            value - the header value
     */
    public void addHeader(String name, String value) {
        responseHeaders.add(new XResponseHeader(name, value));
        super.addHeader(name, value);
    }

    /**
     * Sets a response header with the given name and integer value.
     *
     * @param      name - the name of the header
     *            value - the assigned integer value
     */
    public void setIntHeader(String name, int value) {
        responseHeaders.add(new XResponseHeader(name, String.valueOf(value)));
        super.setHeader(name, Integer.toString(value));
    }

    /**
     * Adds a response header with the given name and integer value.
     *
     * @param      name - the name of the header
     *            value - the assigned integer value
     */
    public void addIntHeader(String name, int value) {
        responseHeaders.add(new XResponseHeader(name, String.valueOf(value)));
        super.addHeader(name, Integer.toString(value));
    }

    /**
     *
     * @deprecated
     */
    private int status = -1;
    private String msg = "";
    public void setStatus(int sc, String msg) {
        status = sc;
        msg = msg;
        super.setStatus(sc);
    }
    public int getStatus() {
        return status;
    }
    public String getMessage() {
        return msg;
    }

    /**
     * Returns the content of the buffered output stream.
     *
     * @return      copy of outputstream as string.
     */
    public String getResponseBody() throws IOException {
        if ( usingStream ) {
            Domain.debug("XHttpServletResponseFacade:getResponseBody() - usingStream");
            return osFacade.getOutputBuffer();
        } else if ( usingWriter ) {
            Domain.debug("XHttpServletResponseFacade:getResponseBody() - usingWriter");
            return (( XServletWriterFacade )writer).getOutputBuffer();
        } else {
            Domain.debug("XHttpServletResponseFacade:getResponseBody() - nor writer nor stream - osFacade = " + osFacade + " writer = " + writer);
            return "";
        }
    }


}
