/*
 * $Header: /home/cvs/jakarta-slide/src/webdav/server/org/apache/slide/webdav/method/PutMethod.java,v 1.17 2001/08/31 03:21:45 remm Exp $
 * $Revision: 1.17 $
 * $Date: 2001/08/31 03:21:45 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.slide.webdav.method;

import java.security.Principal;
import java.io.*;
import java.util.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import javax.servlet.*;
import javax.servlet.http.*;
import org.apache.util.WebdavStatus;
import org.apache.slide.common.*;
import org.apache.slide.structure.*;
import org.apache.slide.content.*;
import org.apache.slide.security.*;
import org.apache.slide.lock.*;
import org.apache.slide.webdav.*;

/**
 * PUT method.
 *
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 */
public class PutMethod extends WebdavMethod {
    
    
    // -------------------------------------------------------------- Constants
    
    
    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * Resource to be written.
     */
    private String resourcePath;
    
    
    // ----------------------------------------------------------- Constructors
    
    
    /**
     * PUT Method constructor.
     *
     * @param token Namespace access token
     * @param req HTTP request
     * @param resp HTTP response
     */
    public PutMethod(NamespaceAccessToken token, HttpServletRequest req,
                     HttpServletResponse resp, WebdavServletConfig config) {
        super(token, req, resp, config);
    }
    
    
    // ------------------------------------------------------ Protected Methods
    
    
    /**
     * Parse XML request.
     *
     * @exception WebdavException Does not happen
     */
    protected void parseRequest()
        throws WebdavException {
        resourcePath = requestUri;
        if (resourcePath == null) {
            resourcePath = "/";
        }
    }
    
    /**
     * Execute request.
     *
     * @exception WebdavException Bad request
     */
    protected void executeRequest()
        throws WebdavException {
        
        // Prevent dirty reads
        slideToken.setForceStoreEnlistment(true);
        
        try {
            
            try {
                
                NodeRevisionDescriptors revisionDescriptors =
                    content.retrieve(slideToken, resourcePath);
                
                NodeRevisionNumber revisionNumber =
                    revisionDescriptors.getLatestRevision();
                NodeRevisionDescriptor oldRevisionDescriptor = null;
                if (revisionNumber != null) {
                    try {
                        oldRevisionDescriptor = content.retrieve
                            (slideToken, revisionDescriptors);
                    } catch (RevisionDescriptorNotFoundException e) {
                    }
                }
                if (WebdavUtils.isCollection(oldRevisionDescriptor)) {
                    resp.setStatus(WebdavStatus.SC_CONFLICT);
                    return;
                }
                
                NodeRevisionDescriptor revisionDescriptor = null;
                if (oldRevisionDescriptor == null) {
                    revisionDescriptor = new NodeRevisionDescriptor();
                } else {
                    revisionDescriptor = oldRevisionDescriptor;
                    revisionDescriptor.setContentLength(-1);
                }
                NodeRevisionContent revisionContent =
                    new NodeRevisionContent();
                //revisionContent.setContent(req.getReader());
                revisionContent.setContent(req.getInputStream());
                
                NodeProperty property = null;
                
                // Get content length
                property = new NodeProperty
                    ("getcontentlength", new Long(req.getContentLength()),
                     true);
                revisionDescriptor.setProperty(property);
                
                // Last modification date
                revisionDescriptor.setLastModified(new Date());
                
                // Etag generation
                String etag = resourcePath.hashCode() + "_"
                    + revisionNumber.hashCode() + "_"
                    + revisionDescriptor.getContentLength();
                property = new NodeProperty("getetag", etag, true);
                revisionDescriptor.setProperty(property);
                
                // Normally assume the 'getcontentlanguage' and
                // 'getcontenttype' are set, however, before we clear the
                // 'resourcetype' need to check for the case when a
                // 'lock-null' is created just before the initial PUT. In
                // that case need to add the missing properties.
                property = revisionDescriptor.getProperty("resourcetype");
                if (((String) property.getValue()).equals("<lock-null/>")) {
                    property =
                        new NodeProperty("getcontentlanguage", "en", true);
                    revisionDescriptor.setProperty(property);
                    
                    String contentType = req.getContentType();
                    if (contentType == null) {
                        contentType = getConfig().getServletContext()
                            .getMimeType(resourcePath);
                    }
                    if (contentType == null) {
                        contentType = getConfig().getDefaultMimeType();
                    }
                    property = new NodeProperty("getcontenttype", contentType,
                                                true);
                    revisionDescriptor.setProperty(property);
                }
                
                // Resource type
                property = new NodeProperty("resourcetype", "", true);
                revisionDescriptor.setProperty(property);
                
                content.create(slideToken, resourcePath, revisionDescriptor,
                               revisionContent);
                
            } catch (LinkedObjectNotFoundException e) {
                // Nothing we can do here ...
                e.printStackTrace();
                //
                // make sure the transaction is aborted
                // throw any WebDAV exception to indicate the transaction wants to be aborted
                //
                throw new WebdavException(WebdavStatus.SC_ACCEPTED, false);
            } catch (ObjectNotFoundException e) {
                
                // Todo : Check to see if parent exists
                SubjectNode subject = new SubjectNode();
                // Creating an object
                structure.create(slideToken, subject, resourcePath);
                
                NodeRevisionDescriptor revisionDescriptor =
                    new NodeRevisionDescriptor(req.getContentLength());
                
                NodeProperty property = null;
                
                // Creation date
                
                // Resource type
                property = new NodeProperty("resourcetype", "", true);
                revisionDescriptor.setProperty(property);
                
                // Source
                property = new NodeProperty("source", "", true);
                revisionDescriptor.setProperty(property);
                
                // Get content language
                property = new NodeProperty("getcontentlanguage", "en", true);
                revisionDescriptor.setProperty(property);
                
                // Get content length
                property = new NodeProperty
                    ("getcontentlength",
                     new Long(req.getContentLength()), true);
                revisionDescriptor.setProperty(property);
                
                // Get content type
                String contentType = req.getContentType();
                if (contentType == null) {
                    contentType = getConfig().getServletContext()
                        .getMimeType(resourcePath);
                }
                if (contentType == null) {
                    contentType = getConfig().getDefaultMimeType();
                }
                property = new NodeProperty("getcontenttype", contentType,
                                            true);
                revisionDescriptor.setProperty(property);
                
                // Last modification date
                revisionDescriptor.setLastModified(new Date());
                
                // Etag generation
                String etag = resourcePath.hashCode() + "_"
                    + (new NodeRevisionNumber()).hashCode() + "_"
                    + req.getContentLength();
                property = new NodeProperty("getetag", etag, true);
                revisionDescriptor.setProperty(property);
                
                if (isMsProprietarySupport()) {
                    
                    // Name
                    /*
                     property = new NodeProperty("name", resourcePath,
                     "d", "DAV");
                     revisionDescriptor.setProperty(property);
                     */
                    
                    // ParentName
                    // TODO : Fix this
                    /*
                     property = new NodeProperty("parentname", resourcePath,
                     "ms", "MICROSOFT");
                     revisionDescriptor.setProperty(property);
                     */
                    
                    // Href
                    /*
                     property = new NodeProperty("href", resourcePath,
                     "ms", "MICROSOFT");
                     revisionDescriptor.setProperty(property);
                     */
                    
                    // Is hidden
                    property = new NodeProperty("ishidden", "0", "MICROSOFT");
                    revisionDescriptor.setProperty(property);
                    
                    // Is collection
                    property = new NodeProperty("iscollection", "0",
                                                "MICROSOFT");
                    revisionDescriptor.setProperty(property);
                    
                    // Is read only
                    property = new NodeProperty("isreadonly", "0",
                                                "MICROSOFT");
                    revisionDescriptor.setProperty(property);
                    
                    // Content class
                    // TODO : Find what it is ...
                    /*
                     property = new NodeProperty("contentclass", "",
                     "MICROSOFT");
                     revisionDescriptor.setProperty(property);
                     */
                    
                    // Last accessed
                    property = new NodeProperty("lastaccessed",
                                                    (new Date()).toString(),
                                                "MICROSOFT");
                    revisionDescriptor.setProperty(property);
                    
                    // Is structured document
                    /*
                     property = new NodeProperty("isstructureddocument",
                     "false", "MICROSOFT");
                     revisionDescriptor.setProperty(property);
                     */
                    
                    // Default document
                    /*
                     property = new NodeProperty("defaultdocument",
                     "", "MICROSOFT");
                     revisionDescriptor.setProperty(property);
                     */
                    
                    // Is root
                    /*
                     property = new NodeProperty("isroot", "false",
                     "MICROSOFT");
                     revisionDescriptor.setProperty(property);
                     */
                    
                }
                
                // Creating revisionDescriptor associated with the object
                NodeRevisionContent revisionContent =
                    new NodeRevisionContent();
                revisionContent.setContent(req.getInputStream());
                
                content.create(slideToken, resourcePath, revisionDescriptor,
                               revisionContent);
                
            }
            
            String status = new String("HTTP/1.1 " + WebdavStatus.SC_CREATED
                                           + " " + WebdavStatus.getStatusText
                                           (WebdavStatus.SC_CREATED));
            
            
        } catch (Exception e) {
            resp.setStatus(getErrorCode(e));  // special handling needed
            throw new WebdavException(WebdavStatus.SC_ACCEPTED, false); // abort the TA
        }
        
        
    }
    
        
        
        
    /**
     * Get return status based on exception type.
     */
    protected int getErrorCode(Exception ex) {
        try {
            throw ex;
        } catch (RevisionAlreadyExistException e) {
            return WebdavStatus.SC_NOT_FOUND;
        } catch (ObjectAlreadyExistsException e) {
            return WebdavStatus.SC_NOT_FOUND;
        } catch (ObjectNotFoundException e) {
            return WebdavStatus.SC_CONFLICT;
        } catch (LinkedObjectNotFoundException e) {
            return WebdavStatus.SC_NOT_FOUND;
        } catch (Exception e) {
            return super.getErrorCode(e);
        }
    }


        
    /**
     * Returns true
     */
    protected boolean methodNeedsTransactionSupport() {
        return true;
    }
    
    
}
